import re
from typing import Tuple

import requests
import requests.auth
from PIL import Image

from dememefy.posts.posts import BasePosts


class RedditPosts(BasePosts):
    _AUTH = "https://www.reddit.com/api/v1/access_token"

    def __init__(self, username: str, password: str, token: str, client_id: str, thread: str = "memes"):
        if not all(isinstance(arg, str) for arg in [username, password, token, client_id, thread]):
            raise TypeError("All credentials must be str")
        self.__username = username
        self.__password = password
        self.__token = token
        self.__client_id = client_id
        self.__thread = thread

        self.__feed = f"https://oauth.reddit.com/r/{thread}/hot?limit=1"
        self.__auth_headers = self.__auth()

    def get_post(self) -> Tuple[str, Image.Image]:
        posts = requests.get(self.__feed, headers=self.__auth_headers).json()["data"]["children"][-1:][0]["data"]  # noqa: E501
        if (posts["title"] is None) or (posts["url"] is None):
            raise ValueError("Can't parse title/url in post")
        self.__feed = f"https://oauth.reddit.com/r/{self.__thread}/hot?limit=1&after={posts['name']}"
        if re.search(r"\.(jpeg|jpg|png)", posts["url"]) is None:
            return self.get_post()
        return (posts["title"], self._download_pic(posts["url"]))  # noqa: E501

    def __auth(self) -> dict:
        auth = requests.auth.HTTPBasicAuth(self.__client_id, self.__token)
        user_agent = {"User-Agent": "Dememefy"}
        res = requests.post(self._AUTH,
                            auth=auth,
                            data={"grant_type": "password",
                                  "username": self.__username,
                                  "password": self.__password},
                            headers=user_agent)
        token = res.json()["access_token"]
        return {**user_agent, **{"Authorization": f"bearer {token}"}}



import io
import sys
from argparse import ArgumentParser, FileType

import toml
from PIL import Image

from dememefy import posts
from dememefy.image import Demotivator

SERVICES = {
    "reddit": (posts.RedditPosts, ["username", "password", "client_id", "token"], ["thread"]),
}


def _parse_args():
    parser = ArgumentParser(
        prog="dememefy", description="Make media fun again")
    parser.add_argument(
        "--service", "-s",
        dest="service",
        type=str,
        choices=SERVICES.keys(),
        help="Target Service.")
    parser.add_argument(
        "--save-to", "-st",
        type=str,
        default="result.png",
        dest="destination",
        help="Destination for demotivator"
    )
    parser.add_argument(
        "--open", "-o",
        action="store_true",
        help="Open output intentionally"
    )
    parser.add_argument(
        "image",
        type=FileType("rb"),
        nargs="?",
        default=sys.stdin.buffer,
        help="Image for demotivator (works only in plain mode).")
    parser.add_argument(
        "--text", "-txt",
        dest="text",
        type=str,
        default="title",
        help="Text for demotivator (works only in plain mode).")
    parser.add_argument(
        "--config", "-c",
        dest="config",
        type=str,
        help="Path to config for parse memes in target service.")
    return parser.parse_args()


def save_demotivator(picture: Image.Image, text: str, filename: str, open: bool):
    demotivator = Demotivator(
        image=picture, text=text, x_start=75, y_start=45)
    picture = demotivator.create()
    picture.save(filename)
    if open:
        try:
            picture.show()
        except OSError as error:
            print(f"Can't open generated output. Error: {error}.")


def main():
    args = _parse_args()

    if not args.service:
        if (args.text is None) or (args.image is None):
            raise ValueError(
                "In plain mode you need to set both text and image.")
        with args.image as image:
            picture = Image.open(io.BytesIO(image.read()))
        save_demotivator(picture, args.text, args.destination, args.open)
        return

    try:
        with open(args.config) as file:
            toml_string = toml.loads(file.read())
    except FileNotFoundError as error:
        raise FileNotFoundError(f"Can't find toml file. Error: {error}.")
    except:
        raise TypeError("Can't parse config file. Perhaps misconfiguration.")

    service_name, credentials, params = SERVICES[args.service]

    kwargs = {}
    for arg_name in [*credentials, *params]:
        try:
            arg_value = toml_string[args.service][arg_name]
            if arg_value:
                kwargs[arg_name] = arg_value
        except KeyError:
            if arg_name in credentials:
                raise KeyError(f"{arg_name} is missing. It's necessary param!")

    service = service_name(**kwargs)

    try:
        amount = toml_string[args.service]["amount"]
    except:
        amount = 1

    for i in range(amount):
        text, picture = service.get_post()
        try:
            filename, extension = args.destination.split(".")
            save_demotivator(picture, text, f"{filename}{i+1}.{extension}", args.open)  # noqa
        except ValueError as error:
            print(f"Destination filename set wrong. Error: {error}")



import io
import re

import requests
from PIL import Image


class BasePosts:
    _AUTH = NotImplemented

    def _download_pic(self, url: str) -> Image.Image:
        if re.search(r"\.(jpeg|jpg|png)", url) is None:
            raise ValueError(
                f"Can't download pic, file has invalid extension. Url: {url}")

        image_bytes = b''
        response = requests.get(url, stream=True)
        for chunk in response:
            image_bytes += chunk

        return Image.open(io.BytesIO(image_bytes))



from typing import Tuple

from PIL import Image

from dememefy.text import Font, Text


class Demotivator:
    def __init__(self, image: Image.Image, text: str, x_start: int, y_start: int):
        self.__image = image
        self.__text = text
        self.__x_start = x_start
        self.__y_start = y_start

    def create(self):
        self.__image = self.__add_border(
            5, (0, 0, 0), self.__image)  # add black border
        self.__image = self.__add_border(
            2, (255, 255, 255), self.__image)  # add white border
        template = Image.new(mode="RGB", size=(
            self.__image.width+(self.__x_start*2), self.__image.height+(self.__y_start*2)+200))
        template.paste(self.__image, self.__get_coords(
            self.__image))  # paste image to template
        Text(Font(font_filename="Symbola.ttf", size=45, font_y=self.__image.height+self.__y_start+70)).draw(
            template, self.__text)  # paste text to template

        return template

    def __get_coords(self, image: Image.Image) -> Tuple[int, int, int, int]:
        return (self.__x_start, self.__y_start, image.width+self.__x_start, image.height+self.__y_start)

    def __add_border(self, size: int, color: Tuple[int, int, int], image: Image.Image) -> Image.Image:
        img_with_border = Image.new("RGB", (image.width+(size*2),
                                            image.height+(size*2)), color)  # type: ignore
        img_with_border.paste(
            image, (size, size, self.__image.width+size, image.height+size))
        return img_with_border



from os.path import dirname, join

from setuptools import find_packages, setup

setup(
    author = "pinktoxin",
    author_email = "pinktoxindev@gmail.com",
    name="dememefy",
    version="1.2.6",
    url="https://github.com/pinktoxin/dememefy",
    description="dememefy - make pics fun again",
    packages=find_packages(),
    python_requires=">=3.7",
    install_requires=[
        "toml",
        "pillow",
        "requests"
    ],
    entry_points={
        "console_scripts": [
            "dememefy = dememefy.cli:main",
        ],
    },
    include_package_data=True,
    package_data={"dememefy": ['fonts/Symbola.ttf']},
)



import os
from dataclasses import dataclass

from PIL import Image, ImageDraw, ImageFont


@dataclass
class Font:
    font_filename: str
    size: int
    font_y: int


class Text:
    def __init__(self, font: Font):
        self.__font = font

    def draw(self, image: Image.Image, text: str):
        font_path = os.path.join(os.path.dirname(
            __file__), "fonts", self.__font.font_filename)
        draw = ImageDraw.Draw(image)
        text_font = ImageFont.truetype(font_path, self.__font.size)
        text_width = text_font.getsize(text)[0]
        while text_width >= image.width - 10*2:
            text_font = ImageFont.truetype(font_path, self.__font.size)
            text_width = text_font.getsize(text)[0]
            self.__font.size -= 1

        draw.text(
            ((image.width - text_width) / 2, self.__font.font_y), text, font=text_font,
        )
